(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     30241,        701]
NotebookOptionsPosition[     29884,        685]
NotebookOutlinePosition[     30262,        701]
CellTagsIndexPosition[     30219,        698]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[StyleBox["Plenum <--> Cambium conversions using Mathematica \
transformation rules.  \n\nAuthors: Darren Davis (translator code), Guy \
Yosiphon (grammar examples), Scientific Inference Systems Laboratory, UC \
Irvine\n\nUsage:\n\tPlenum --> Cambium translation: \n\t\tcambiumVersion = \
plenumVersion //. TranslateP2C\n\tCambium --> Plenum translation:\n\t\t\
plenumVersion = cambiumVersion //. TranslateC2P\n\nIf the input form isn't \
recognized (bad syntax or unsupported Plenum/Cambium feature), the result \
will be the same as the input.",
 FontSize->14]], "Text",
 CellChangeTimes->{{3.4601369649309998`*^9, 3.4601371225690002`*^9}, {
   3.460137192552*^9, 3.46013736488*^9}, {3.4602328925863633`*^9, 
   3.4602328945673633`*^9}, {3.4921814938008575`*^9, 
   3.4921815305059566`*^9}, {3.492190244167014*^9, 3.4921902698374825`*^9}, 
   3.492197120670328*^9, {3.4921974119739895`*^9, 3.492197413022049*^9}},
 Background->GrayLevel[0.85]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Only", " ", "TranslateP2C", " ", "and", " ", "TranslateC2P", " ", "need", 
    " ", "to", " ", "be", " ", "used", " ", 
    RowBox[{"directly", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Results", " ", "in", " ", "the", " ", "set", " ", "of", " ", "rules", 
     " ", "referenced", " ", "by", " ", "the", " ", "rules"}], " ", "\[Rule]",
     " ", 
    RowBox[{
     RowBox[{"{", "...", "}"}], " ", "element", " ", "of", " ", "the", " ", 
     "Plenum", " ", "grammar"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"ExtractRules", " ", "=", " ", 
     RowBox[{
      RowBox[{"Grammar", "[", 
       RowBox[{"___", ",", " ", 
        RowBox[{"rules", "\[Rule]", " ", "rules_"}], ",", " ", "___"}], "]"}],
       " ", "\[Rule]", " ", "rules"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Results", " ", "in", " ", "everything", " ", "within", " ", "the", " ", 
     "Grammar", " ", "head", " ", "that", " ", 
     RowBox[{"isn", "'"}], "t", " ", "part", " ", "of", " ", "the", " ", 
     "rules", " ", "element"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ExtractNonruleInfo", " ", "=", " ", 
     RowBox[{
      RowBox[{"Grammar", "[", 
       RowBox[{
        RowBox[{"pre", ":", " ", 
         RowBox[{"Longest", "[", 
          RowBox[{
           RowBox[{"Except", "[", 
            RowBox[{"rules", " ", "\[Rule]", " ", "_"}], "]"}], "..."}], 
          "]"}]}], ",", " ", 
        RowBox[{"rules", "\[Rule]", " ", "rules_"}], ",", " ", 
        RowBox[{"post", ":", " ", 
         RowBox[{"Longest", "[", 
          RowBox[{
           RowBox[{"Except", "[", 
            RowBox[{"rules", " ", "\[Rule]", " ", "_"}], "]"}], "..."}], 
          "]"}]}]}], "]"}], " ", "\[Rule]", " ", 
      RowBox[{"GrammarParameters", "[", 
       RowBox[{"pre", ",", " ", "post"}], "]"}]}]}], " ", ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Rule", "-", "by", "-", 
     RowBox[{
     "rule", " ", "translation", " ", "from", " ", "Plenum", " ", "grammar", 
      " ", "rules", " ", "to", " ", "Cambium", " ", 
      RowBox[{"arrows", ".", "  ", "Each"}], " ", "Plenum", " ", "rule", " ", 
      "will", " ", "be", " ", "transformed", " ", "into", " ", "exactly", " ",
       "one", " ", "Cambium", " ", 
      RowBox[{"arrow", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"RuleTranslateP2C", " ", "=", " ", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "transforms", " ", "rules", " ", "containing", " ", "a", " ", "With", 
        " ", "clause", " ", "but", " ", "not", " ", "a", " ", "Solving", " ", 
        RowBox[{"clause", "."}]}], "  ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"pre___", ",", " ", 
          RowBox[{"lhs_", " ", "\[Rule]", " ", "rhs_"}], ",", " ", 
          RowBox[{"preWith", ":", " ", 
           RowBox[{"Longest", "[", 
            RowBox[{
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"solving", "[", "__", "]"}], "|", " ", 
                 RowBox[{"with", "[", "___", "]"}], " ", "|", "  ", 
                 RowBox[{"(", 
                  RowBox[{"_", " ", "\[Rule]", " ", "_"}], " ", ")"}], "|", 
                 " ", 
                 RowBox[{"CambiumArrow", "[", "___", "]"}]}], ")"}], ",", " ", 
               RowBox[{"ratePre_", "[", "___", "]"}]}], "]"}], "..."}], 
            "]"}]}], ",", " ", 
          RowBox[{"withClause", ":", " ", 
           RowBox[{"with", "[", "_", "]"}]}], ",", " ", 
          RowBox[{"postWith", ":", " ", 
           RowBox[{"Longest", "[", 
            RowBox[{
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"solving", "[", "__", "]"}], " ", "|", " ", 
                 RowBox[{"(", 
                  RowBox[{"_", " ", "\[Rule]", " ", "_"}], " ", ")"}], "|", 
                 " ", 
                 RowBox[{"CambiumArrow", "[", "___", "]"}]}], ")"}], ",", " ", 
               RowBox[{"ratePost_", "[", "___", "]"}]}], "]"}], "..."}], 
            "]"}]}], ",", " ", "post___"}], "}"}], " ", "\[RuleDelayed]", " ", 
        RowBox[{"{", 
         RowBox[{"pre", ",", " ", 
          RowBox[{"CambiumArrow", "[", 
           RowBox[{
            RowBox[{"DynGram", "[", 
             RowBox[{"discrete", ",", " ", "rule"}], "]"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "lhs", "}"}], "]"}], ",", " ", 
            RowBox[{"{", "}"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "rhs", "}"}], "]"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{
             "preWith", ",", " ", "withClause", ",", " ", "postWith"}], 
             "}"}]}], "]"}], ",", " ", "post"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "transforms", " ", "rules", " ", "containing", " ", "a", " ", 
         "Solving", " ", "clause", " ", "but", " ", "not", " ", "a", " ", 
         "With", " ", "clause"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"pre___", ",", " ", 
          RowBox[{"lhs_", " ", "\[Rule]", " ", "rhs_"}], ",", " ", 
          RowBox[{"preSolving", ":", " ", 
           RowBox[{"Longest", "[", 
            RowBox[{
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"with", "[", "__", "]"}], " ", "|", " ", 
                 RowBox[{"solving", "[", "___", "]"}], " ", "|", " ", 
                 RowBox[{"(", 
                  RowBox[{"_", " ", "\[Rule]", " ", "_"}], ")"}], " ", "|", 
                 " ", 
                 RowBox[{"CambiumArrow", "[", "___", "]"}]}], ")"}], ",", " ", 
               RowBox[{"ratePre_", "[", "___", "]"}]}], "]"}], "..."}], 
            "]"}]}], ",", " ", 
          RowBox[{"solvingClause", ":", " ", 
           RowBox[{"solving", "[", "__", "]"}]}], ",", " ", 
          RowBox[{"postSolving", ":", " ", 
           RowBox[{"Longest", "[", 
            RowBox[{
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"with", "[", "_", "]"}], " ", "|", " ", 
                 RowBox[{"(", 
                  RowBox[{"_", " ", "\[Rule]", " ", "_"}], " ", ")"}], "|", 
                 " ", 
                 RowBox[{"CambiumArrow", "[", "___", "]"}]}], ")"}], ",", " ", 
               RowBox[{"ratePost_", "[", "___", "]"}]}], "]"}], "..."}], 
            "]"}]}], ",", " ", "post___"}], "}"}], " ", "\[RuleDelayed]", " ", 
        RowBox[{"{", 
         RowBox[{"pre", ",", " ", 
          RowBox[{"CambiumArrow", "[", 
           RowBox[{
            RowBox[{"DynGram", "[", 
             RowBox[{"continuous", ",", " ", "rule"}], "]"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "lhs", "}"}], "]"}], ",", " ", 
            RowBox[{"{", "}"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "rhs", "}"}], "]"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{
             "preSolving", ",", " ", "solvingClause", ",", " ", 
              "postSolving"}], "}"}]}], "]"}], ",", " ", "post"}], "}"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "Transforms", " ", "rules", " ", "with", " ", "rate", " ", 
          "clauses"}], ",", " ", 
         RowBox[{
         "where", " ", "no", " ", "such", " ", "rate", " ", "clause", " ", 
          "is", " ", "With", " ", "or", " ", "Solving"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"pre___", ",", " ", 
          RowBox[{"lhs_", " ", "\[Rule]", " ", "rhs_"}], ",", " ", 
          RowBox[{"rateClauses", " ", ":", " ", 
           RowBox[{"Longest", "[", 
            RowBox[{
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"with", "[", "__", "]"}], " ", "|", " ", 
                 RowBox[{"solving", "[", "___", "]"}], " ", "|", " ", 
                 RowBox[{"(", 
                  RowBox[{"_", " ", "\[Rule]", " ", "_"}], ")"}], " ", "|", 
                 " ", 
                 RowBox[{"CambiumArrow", "[", "___", "]"}]}], ")"}], ",", " ", 
               RowBox[{"_", "[", "___", "]"}]}], "]"}], "..."}], "]"}]}], ",",
           " ", "post___"}], "}"}], " ", "\[RuleDelayed]", " ", 
        RowBox[{"{", 
         RowBox[{"pre", ",", " ", 
          RowBox[{"CambiumArrow", "[", 
           RowBox[{
            RowBox[{"DynGram", "[", "rule", "]"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "lhs", "}"}], "]"}], ",", " ", 
            RowBox[{"{", "}"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "rhs", "}"}], "]"}], ",", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"{", "rateClauses", "}"}], "]"}]}], "]"}], ",", " ", 
          "post"}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "Transforms", " ", "rules", " ", "with", " ", "no", " ", "rate", " ",
           "clauses", " ", "at", " ", 
          RowBox[{"all", ".", "  ", "This"}], " ", "transformation", " ", 
          "must", " ", "have", " ", "a", " ", "lower", " ", "precedence", " ",
           "than", " ", "the", " ", "others"}], ",", " ", 
         RowBox[{
         "so", " ", "it", " ", "is", " ", "last", " ", "in", " ", "the", " ", 
          RowBox[{"list", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"lhs_", " ", "\[Rule]", " ", "rhs_"}], ")"}], " ", 
        "\[RuleDelayed]", " ", 
        RowBox[{"CambiumArrow", "[", 
         RowBox[{
          RowBox[{"DynGram", "[", "rule", "]"}], ",", " ", 
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "lhs", "}"}], "]"}], ",", " ", 
          RowBox[{"{", "}"}], ",", " ", 
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "rhs", "}"}], "]"}], ",", " ", 
          RowBox[{"{", "}"}]}], "]"}]}]}], " ", "\[IndentingNewLine]", 
      "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Rule", "-", "by", "-", 
     RowBox[{
     "rule", " ", "translation", " ", "from", " ", "Cambium", " ", "arrows", 
      " ", "to", " ", "Plenum", " ", "grammar", " ", 
      RowBox[{"rules", ".", "  ", "Each"}], " ", "Cambium", " ", "arrow", " ",
       "will", " ", "transformed", " ", "into", " ", "exactly", " ", "one", 
      " ", "Plenum", " ", 
      RowBox[{"rule", ".", " ", "The"}], " ", "MHS", " ", "should", " ", 
      "already", " ", "be", " ", "emptied", " ", "using", " ", 
      "contantTermsFromMhs"}], " ", "-", " ", 
     RowBox[{"TranslateC2P", " ", "takes", " ", "care", " ", "of", " ", 
      RowBox[{"this", "."}]}]}], "  ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"RuleTranslateC2P", " ", "=", " ", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "transforms", " ", "rules", " ", "without", " ", "rate", " ", 
        "clauses"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"CambiumArrow", "[", 
          RowBox[{
           RowBox[{"DynGram", "[", 
            RowBox[{"___", ",", " ", "rule", ",", " ", "___"}], "]"}], ",", 
           " ", "lhs_", ",", " ", 
           RowBox[{"{", "}"}], ",", " ", "rhs_", ",", " ", 
           RowBox[{"{", "}"}]}], "]"}], ")"}], " ", "\[RuleDelayed]", " ", 
        RowBox[{"(", 
         RowBox[{"lhs", " ", "\[Rule]", " ", "rhs"}], ")"}]}], ",", " ", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "transforms", " ", "rules", " ", "with", " ", "rate", " ", 
         "clauses"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"pre___", ",", " ", 
          RowBox[{"(", 
           RowBox[{"CambiumArrow", "[", 
            RowBox[{
             RowBox[{"DynGram", "[", 
              RowBox[{"___", ",", " ", "rule", ",", " ", "___"}], "]"}], ",", 
             " ", "lhs_", ",", " ", 
             RowBox[{"{", "}"}], ",", " ", "rhs_", ",", " ", 
             RowBox[{"{", 
              RowBox[{"rateInfo", ":", " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"with", "[", "_", "]"}], " ", "|", " ", 
                  RowBox[{"solving", "[", "__", "]"}], " ", "|", " ", 
                  RowBox[{"subjectTo", "[", "__", "]"}]}], ")"}], ".."}]}], 
              "}"}]}], "]"}], ")"}], ",", " ", "post___"}], "}"}], " ", 
        "\[RuleDelayed]", " ", 
        RowBox[{"{", 
         RowBox[{"pre", ",", " ", 
          RowBox[{"lhs", " ", "\[Rule]", " ", "rhs"}], ",", " ", "rateInfo", 
          ",", " ", "post"}], "}"}]}], ",", " ", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "if", " ", "single", " ", "items", " ", "appear", " ", "on", " ", 
          "the", " ", "LHS", " ", "or", " ", "RHS", " ", "of", " ", "a", " ", 
          "rule"}], ",", " ", 
         RowBox[{
         "they", " ", "can", " ", "appear", " ", "inside", " ", "a", " ", 
          RowBox[{"(", "singleton", ")"}], " ", "list", " ", "or", " ", 
          "without", " ", "a", " ", 
          RowBox[{"list", ".", "  ", "These"}], " ", "transformations", " ", 
          "enforce", " ", "output", " ", "consistency", " ", "by", " ", 
          "outputting", " ", "all", " ", "single", " ", "items", " ", 
          "without", " ", "a", " ", "list"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"{", "singleton_", "}"}], " ", "\[Rule]", " ", "rhs_"}], 
         ")"}], "\[Rule]", " ", 
        RowBox[{"(", 
         RowBox[{"singleton", " ", "\[Rule]", " ", "rhs"}], ")"}]}], ",", " ",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"lhs_", " ", "\[Rule]", " ", 
          RowBox[{"{", "singleton_", "}"}]}], ")"}], " ", "\[Rule]", " ", 
        RowBox[{"(", 
         RowBox[{"lhs", " ", "\[Rule]", " ", "singleton"}], ")"}]}]}], 
      "\[IndentingNewLine]", "}"}]}], " ", ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{
     "Cambium", " ", "has", " ", "a", " ", "MHS", " ", "but", " ", "Plenum", 
      " ", 
      RowBox[{"doesn", "'"}], "t"}], ",", " ", 
     RowBox[{
     "so", " ", "when", " ", "translating", " ", "from", " ", "Plenum", " ", 
      "to", " ", "Cambium"}], ",", " ", 
     RowBox[{
      RowBox[{"any", " ", 
       RowBox[{"LHS", "/", "RHS"}]}], "-", 
      RowBox[{
      "identical", " ", "elements", " ", "can", " ", "be", " ", "put", " ", 
       "in", " ", "the", " ", 
       RowBox[{"MHS", "."}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"constantTermsToMhs", " ", "=", " ", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"CambiumArrow", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"dgType", ":", " ", 
          RowBox[{"Except", "[", 
           RowBox[{
            RowBox[{"DynGram", "[", 
             RowBox[{"___", ",", " ", 
              RowBox[{"(", 
               RowBox[{"continuous", " ", "|", " ", "composite"}], ")"}], ",",
               " ", "___"}], "]"}], ",", " ", 
            RowBox[{"DynGram", "[", 
             RowBox[{"___", ",", " ", "discrete", ",", " ", "___"}], "]"}]}], 
           "]"}]}], ")"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
         "lhsPre___", ",", " ", "identicalElt_", ",", " ", "lhsPost___"}], 
         "}"}], ",", " ", "mhs_List", ",", " ", 
        RowBox[{"{", 
         RowBox[{
         "rhsPre___", ",", " ", "identicalElt_", ",", " ", "rhsPost___"}], 
         "}"}], ",", " ", 
        RowBox[{"(", 
         RowBox[{"clauses", ":", " ", 
          RowBox[{"Except", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"___", ",", " ", 
              RowBox[{"solving", "[", "___", "]"}], ",", " ", "___"}], "}"}], 
            ",", " ", 
            RowBox[{"{", 
             RowBox[{"___", ",", " ", 
              RowBox[{"with", "[", "___", "]"}], ",", " ", "___"}], "}"}]}], 
           "]"}]}], ")"}]}], "]"}], " ", "\[RuleDelayed]", " ", 
      "\[IndentingNewLine]", 
      RowBox[{"CambiumArrow", "[", 
       RowBox[{"dgType", ",", " ", 
        RowBox[{"{", 
         RowBox[{"lhsPre", ",", " ", "lhsPost"}], "}"}], ",", " ", 
        RowBox[{"Prepend", "[", 
         RowBox[{"mhs", ",", " ", "identicalElt"}], "]"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{"rhsPre", ",", " ", "rhsPost"}], "}"}], ",", " ", 
        "clauses"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Since", " ", "Plenum", " ", 
      RowBox[{"doesn", "'"}], "t", " ", "have", " ", "a", " ", "MHS"}], ",", 
     " ", 
     RowBox[{
     "all", " ", "MHS", " ", "elements", " ", "should", " ", "appear", " ", 
      "as", " ", "identical", " ", "copies", " ", "on", " ", "the", " ", 
      "LHS", " ", "and", " ", "RHS"}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"constantTermsFromMhs", " ", "=", " ", 
     RowBox[{
      RowBox[{"CambiumArrow", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"dgType", ":", " ", 
          RowBox[{"Except", "[", 
           RowBox[{
            RowBox[{"DynGram", "[", 
             RowBox[{"___", ",", " ", 
              RowBox[{"(", 
               RowBox[{"continuous", " ", "|", " ", "composite"}], ")"}], ",",
               " ", "___"}], "]"}], ",", " ", 
            RowBox[{"DynGram", "[", 
             RowBox[{"___", ",", " ", "discrete", ",", " ", "___"}], "]"}]}], 
           "]"}]}], ")"}], ",", " ", "lhs_", ",", " ", 
        RowBox[{"(", 
         RowBox[{"mhs", ":", " ", 
          RowBox[{"{", "__", "}"}]}], ")"}], ",", " ", "rhs_", ",", " ", 
        RowBox[{"(", 
         RowBox[{"clauses", ":", " ", 
          RowBox[{"Except", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"___", ",", " ", 
              RowBox[{"solving", "[", "___", "]"}], ",", " ", "___"}], "}"}], 
            ",", " ", 
            RowBox[{"{", 
             RowBox[{"___", ",", " ", 
              RowBox[{"with", "[", "___", "]"}], ",", " ", "___"}], "}"}]}], 
           "]"}]}], ")"}]}], "]"}], " ", "\[RuleDelayed]", " ", 
      RowBox[{"CambiumArrow", "[", 
       RowBox[{"dgType", ",", " ", 
        RowBox[{"Flatten", "[", 
         RowBox[{"Prepend", "[", 
          RowBox[{"lhs", ",", " ", "mhs"}], "]"}], "]"}], ",", " ", 
        RowBox[{"{", "}"}], ",", " ", 
        RowBox[{"Flatten", "[", 
         RowBox[{"Prepend", "[", 
          RowBox[{"rhs", ",", " ", "mhs"}], "]"}], "]"}], ",", " ", 
        "clauses"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"Overall", " ", "Plenum"}], "<", 
      RowBox[{"--", 
       RowBox[{">", 
        RowBox[{"Cambium", " ", "translations"}]}]}]}], ",", " ", 
     RowBox[{
      RowBox[{
      "and", " ", "the", " ", "only", " ", "transformations", " ", "that", 
       " ", "need", " ", "to", " ", "be", " ", "used", " ", 
       RowBox[{"directly", ".", "  ", "These"}], " ", "transformations", " ", 
       "ensure", " ", "proper", " ", "ordering", " ", "and", " ", 
       "application", " ", "of", " ", "the", " ", "lower"}], "-", 
      RowBox[{
      "level", " ", "transformations", " ", "defined", " ", "above"}]}]}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Plenum", " ", "to", " ", "Cambium"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"TranslateP2C", " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"grammar", ":", " ", 
        RowBox[{"Grammar", "[", "___", "]"}]}], ")"}], " ", "\[RuleDelayed]", 
      " ", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "overall", " ", "container", " ", "for", " ", "individual", " ", 
        "reaction", " ", "arrows", " ", "is", " ", "a", " ", "composite", " ",
         "arrow"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"CambiumArrow", "[", 
       RowBox[{
        RowBox[{"DynGram", "[", "composite", "]"}], ",", " ", 
        RowBox[{"{", "}"}], ",", " ", 
        RowBox[{"{", "}"}], ",", " ", 
        RowBox[{"{", "}"}], ",", " ", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{"all", " ", "non"}], "-", 
          RowBox[{
          "rule", " ", "Grammar", " ", "elements", " ", "put", " ", "inside", 
           " ", "a", " ", "GrammarParameters", " ", "head"}]}], " ", "*)"}], 
        " ", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"grammar", " ", "/.", " ", "ExtractNonruleInfo"}], ")"}], 
          ",", " ", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
            RowBox[{"rule", "-", "by", "-", 
             RowBox[{
             "rule", " ", "translation", " ", "is", " ", "applied", " ", "to",
               " ", "the", " ", 
              RowBox[{"grammar", "'"}], "s", " ", "rule", " ", "list"}]}], 
            ",", " ", 
            RowBox[{
             RowBox[{"then", " ", 
              RowBox[{"Cambium", "/", "non"}]}], "-", 
             RowBox[{
             "Plenum", " ", "features", " ", "are", " ", "exploited", " ", 
              RowBox[{"(", 
               RowBox[{
               "currently", " ", "just", " ", "unchanged", " ", "terms", " ", 
                "in", " ", "the", " ", "MHS"}], ")"}]}]}]}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"grammar", " ", "/.", " ", "ExtractRules"}], " ", "//.",
               " ", "RuleTranslateP2C"}], " ", "//.", " ", 
             "constantTermsToMhs"}], ")"}], " ", "//.", " ", 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "then", " ", "put", " ", "the", " ", "rules", " ", "in", " ", "a",
              " ", "Rules", " ", "header", " ", "rather", " ", "than", " ", 
             "a", " ", "List", " ", "header"}], " ", "*)"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"List", "[", 
              RowBox[{"arrows", ":", " ", 
               RowBox[{"Longest", "[", 
                RowBox[{
                 RowBox[{"CambiumArrow", "[", "__", "]"}], ".."}], "]"}]}], 
              "]"}], " ", "\[Rule]", " ", 
             RowBox[{"Rules", "[", "arrows", "]"}]}], ")"}]}]}], "}"}]}], " ",
        "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Cambium", " ", "to", " ", "Plenum"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"TranslateC2P", " ", "=", " ", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "recognize", " ", "the", " ", "container", " ", "as", " ", "a", " ", 
        "Cambium", " ", "arrow", " ", "with", " ", "DynGram", " ", "type", 
        " ", "and", " ", "a", " ", "composite"}], "-", "attribute"}], " ", 
      "*)"}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"CambiumArrow", "[", 
        RowBox[{
         RowBox[{"DynGram", "[", 
          RowBox[{"___", ",", " ", "composite", ",", " ", "___"}], "]"}], ",",
          " ", "_", ",", " ", "_", ",", " ", "_", ",", " ", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{
           "match", " ", "and", " ", "name", " ", "the", " ", "parameters"}], 
           ",", " ", 
           RowBox[{"if", " ", "any"}]}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"parameters", ":", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"GrammarParameters", "[", "params___", "]"}], " ", "|", 
              " ", 
              RowBox[{"PatternSequence", "[", "]"}]}], ")"}]}], ",", " ", 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "match", " ", "and", " ", "name", " ", "the", " ", "reaction", 
             " ", "arrows"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{"(", 
            RowBox[{"arrows", ":", " ", 
             RowBox[{"Rules", "[", 
              RowBox[{
               RowBox[{"CambiumArrow", "[", 
                RowBox[{
                 RowBox[{"DynGram", "[", 
                  RowBox[{"___", ",", " ", "rule", ",", " ", "___"}], "]"}], 
                 ",", " ", "_", ",", " ", "_", ",", " ", "_", ",", " ", "_"}],
                 "]"}], ".."}], "]"}]}], ")"}]}], "}"}]}], " ", "]"}], ")"}], 
      " ", "\[RuleDelayed]", " ", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{
        "put", " ", "the", " ", "parameters", " ", "at", " ", "the", " ", 
         RowBox[{"beginning", ".", "  ", "For"}], " ", "the", " ", "rules"}], 
        ",", " ", 
        RowBox[{"change", " ", "back", " ", "to", " ", "a", " ", "List", " ", 
         RowBox[{"(", 
          RowBox[{
          "from", " ", "the", " ", "Rules", " ", "header", " ", "in", " ", 
           "Cambium"}], ")"}]}], ",", " ", 
        RowBox[{
         RowBox[{"remove", " ", "any", " ", 
          RowBox[{"Cambium", "/", "non"}]}], "-", 
         RowBox[{"Plenum", " ", "features", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"right", " ", "now"}], ",", " ", 
            RowBox[{
            "this", " ", "is", " ", "just", " ", "copying", " ", "MHS", " ", 
             "terms", " ", "to", " ", "the", " ", "LHS", " ", "and", " ", 
             "RHS"}]}], ")"}]}]}], ",", " ", 
        RowBox[{
         RowBox[{"then", " ", "apply", " ", "rule"}], "-", "by", "-", 
         RowBox[{"rule", " ", "translation"}]}]}], " ", "*)"}], 
      RowBox[{"Grammar", "[", 
       RowBox[{"params", ",", " ", 
        RowBox[{"rules", " ", "\[Rule]", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"arrows", " ", "//.", 
             RowBox[{"(", 
              RowBox[{"Rules", " ", "\[Rule]", " ", "List"}], ")"}]}], " ", "//.",
             " ", "constantTermsFromMhs"}], " ", "//.", " ", 
           "RuleTranslateC2P"}], " ", ")"}]}]}], "]"}]}]}], 
    ";"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.459614611576167*^9, 3.4596146118101673`*^9}, {
   3.4596146529629674`*^9, 3.4596146606537676`*^9}, {3.4596146912765675`*^9, 
   3.459614691978567*^9}, {3.4596147441761675`*^9, 3.4596147457673674`*^9}, {
   3.4596148005233674`*^9, 3.4596148007417674`*^9}, {3.4596148916585674`*^9, 
   3.4596148934057674`*^9}, {3.459614932354967*^9, 3.4596149328229675`*^9}, {
   3.4596156797853675`*^9, 3.4596156801129675`*^9}, {3.4601375710550003`*^9, 
   3.4601377993640003`*^9}, {3.460137841865*^9, 3.4601378977019997`*^9}, {
   3.460137932068*^9, 3.4601380582390003`*^9}, {3.460138105016*^9, 
   3.460138148175*^9}, {3.4601381859630003`*^9, 3.4601383104849997`*^9}, {
   3.460138348267*^9, 3.4601383995439997`*^9}, {3.4601384396940002`*^9, 
   3.4601385110039997`*^9}, {3.460138543645*^9, 3.460138595769*^9}, {
   3.4601386918*^9, 3.460138824977*^9}, {3.460138874398*^9, 
   3.4601389714969997`*^9}, {3.460139046992*^9, 3.4601391703900003`*^9}, {
   3.46013920321*^9, 3.460139242585*^9}, {3.4601392801940002`*^9, 
   3.460139583619*^9}, {3.460143194137*^9, 3.460143260909*^9}, {
   3.460143296171*^9, 3.460143470132*^9}, {3.460143515538*^9, 
   3.460143570391*^9}, {3.4601436911689997`*^9, 3.460143699884*^9}, {
   3.460143833833*^9, 3.460143879672*^9}, {3.4602332320503635`*^9, 
   3.460233233671363*^9}, 3.492188325185255*^9}]
},
AutoGeneratedPackage->Automatic,
WindowSize->{1044, 549},
WindowMargins->{{8, Automatic}, {Automatic, 46}},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (November 10, 2008)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 958, 14, 236, "Text"],
Cell[1506, 36, 28374, 647, 1632, "Input",
 InitializationCell->True]
}
]
*)

(* End of internal cache information *)
